package codechicken.enderstorage.common;

import java.util.Iterator;

import codechicken.core.featurehack.GameDataManipulator;
import codechicken.core.inventory.InventoryUtils;
import codechicken.enderstorage.EnderStorage;
import codechicken.enderstorage.api.EnderStorageManager;
import codechicken.enderstorage.storage.item.ItemEnderChestDummy;

import cpw.mods.fml.common.registry.GameRegistry;

import net.minecraft.block.Block;
import net.minecraft.item.crafting.CraftingManager;
import net.minecraft.item.crafting.IRecipe;
import net.minecraft.inventory.InventoryCrafting;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.world.World;
import net.minecraftforge.oredict.OreDictionary;

public class EnderStorageRecipe implements IRecipe
{
    private static boolean init;

    @Override
    public boolean matches(InventoryCrafting inventorycrafting, World var2)
    {
        for(int row = 0; row <= 1; row++)
            if(offsetMatchesDyes(inventorycrafting, 0, row))
                return true;
        
        return false;
    }
    
    private boolean offsetMatchesDyes(InventoryCrafting inventorycrafting, int col, int row)
    {
        if(!stackMatchesID(inventorycrafting.getStackInRowAndColumn(col+1, row+1), EnderStorage.blockEnderChest.blockID))
            return false;
        
        if(getDyeType(inventorycrafting.getStackInRowAndColumn(col, row)) >= 0)
            return true;
        if(getDyeType(inventorycrafting.getStackInRowAndColumn(col+1, row)) >= 0)
            return true;
        if(getDyeType(inventorycrafting.getStackInRowAndColumn(col+2, row)) >= 0)
            return true;
        
        return false;
    }
    
    public static boolean stackMatchesID(ItemStack stack, int ID)
    {
        return stack != null && stack.itemID == ID;
    }

    public ItemStack getCraftingResult(InventoryCrafting ic)
    {
        for(int row = 0; row <= 1; row++)
        {
            if(!offsetMatchesDyes(ic, 0, row))
                continue;
            
            ItemStack freqowner = ic.getStackInRowAndColumn(1, row+1);
            int freq = freqowner.getItemDamage() & 0xFFF;
            
            int colour1 = recolour(0, row, freq, ic);
            int colour2 = recolour(1, row, freq, ic);
            int colour3 = recolour(2, row, freq, ic);
            
            ItemStack result = InventoryUtils.copyStack(freqowner, 1);
            result.setItemDamage(EnderStorageManager.getFreqFromColours(colour1, colour2, colour3) | freqowner.getItemDamage() & 0xF000);
            return result;
        }
        return null;
    }

    private int recolour(int i, int row, int freq, InventoryCrafting ic)
    {
        int dyeType = getDyeType(ic.getStackInRowAndColumn(i, row));
        if(dyeType >= 0)
            return ~dyeType&0xF;
        return EnderStorageManager.getColourFromFreq(freq, i);
    }

    public int getRecipeSize()
    {
        return 6;
    }
    
    public ItemStack getRecipeOutput()
    {
        return new ItemStack(EnderStorage.blockEnderChest);
    }
    
    public static void init()
    {
        if(init)return;
        init = true;
        
        EnderStorageRecipe instance = new EnderStorageRecipe();
        GameRegistry.addRecipe(instance);
        addNormalRecipies();
        if(EnderStorage.disableVanillaEnderChest)
            removeVanillaChest();
    }
    
    private static void removeVanillaChest()
    {
        Item.itemsList[Block.enderChest.blockID] = null;
        GameDataManipulator.createHiddenItem(new Runnable()
        {            
            @SuppressWarnings("unused")
            @Override
            public void run()
            {
                new ItemEnderChestDummy(Block.enderChest.blockID-256);
            }
        });
        Iterator<IRecipe> iterator = CraftingManager.getInstance().getRecipeList().iterator();
        while(iterator.hasNext())
        {
            ItemStack r = iterator.next().getRecipeOutput();
            if(r != null && r.itemID == Block.enderChest.blockID)
                iterator.remove();
        }
    }

    private static void addNormalRecipies()
    {
        for(int i = 0; i < 16; i++)
        {
            GameRegistry.addRecipe(new ItemStack(EnderStorage.blockEnderChest, 1, EnderStorageManager.getFreqFromColours(i, i, i)), new Object[]{
                "bWb",
                "OCO",
                "bpb",
                'b', Item.blazeRod,
                'p', Item.enderPearl,
                'O', Block.obsidian,
                'C', Block.chest,
                'W', new ItemStack(Block.cloth, 1, i)});

            GameRegistry.addRecipe(new ItemStack(EnderStorage.itemEnderPouch, 1, EnderStorageManager.getFreqFromColours(i, i, i)), new Object[]{
                "blb",
                "lpl",
                "bWb",
                'b', Item.blazePowder,
                'p', Item.enderPearl,
                'l', Item.leather,
                'W', new ItemStack(Block.cloth, 1, i)});
            
            GameRegistry.addRecipe(new ItemStack(EnderStorage.blockEnderChest, 1, 1<<12 | EnderStorageManager.getFreqFromColours(i, i, i)), new Object[]{
                "bWb",
                "OCO",
                "bpb",
                'b', Item.blazeRod,
                'p', Item.enderPearl,
                'O', Block.obsidian,
                'C', Item.cauldron,
                'W', new ItemStack(Block.cloth, 1, i)});
        }
    }
    
    public static int getDyeType(ItemStack item)
    {
        if(item == null)return -1;
        if(item.itemID == Item.dyePowder.itemID)
            return item.getItemDamage();
        for(int i = 0; i < 16; i++)
        {
            for(ItemStack target : OreDictionary.getOres(oreDictionaryNames[i]))
                if(OreDictionary.itemMatches(target, item, false))
                    return i;
        }
        return -1;
    }
    
    public static String[] oreDictionaryNames = new String[]{
        "dyeBlack",
        "dyeRed",
        "dyeGreen",
        "dyeBrown",
        "dyeBlue",
        "dyePurple",
        "dyeCyan",
        "dyeLightGray",
        "dyeGray",
        "dyePink",
        "dyeLime",
        "dyeYellow",
        "dyeLightBlue",
        "dyeMagenta",
        "dyeOrange",
        "dyeWhite"};
}
