package codechicken.enderstorage.storage.item;

import codechicken.core.ClientUtils;
import codechicken.core.IGuiPacketSender;
import codechicken.core.ServerUtils;
import codechicken.core.inventory.InventoryUtils;
import codechicken.core.packet.PacketCustom;
import codechicken.enderstorage.api.AbstractEnderStorage;
import codechicken.enderstorage.api.EnderStorageManager;
import codechicken.enderstorage.internal.EnderStorageSPH;
import cpw.mods.fml.relauncher.Side;
import cpw.mods.fml.relauncher.SideOnly;

import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.entity.player.EntityPlayerMP;
import net.minecraft.entity.player.InventoryPlayer;
import net.minecraft.inventory.IInventory;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.nbt.NBTTagList;

import static codechicken.enderstorage.storage.EnderItemStoragePlugin.*;

public class EnderItemStorage extends AbstractEnderStorage implements IInventory
{
    private ItemStack[] items;
    private int open;
    private int size;

    public EnderItemStorage(EnderStorageManager manager, String owner, int freq)
    {
        super(manager, owner, freq);
        size = configSize;
        empty();
    }
    
    public void loadFromTag(NBTTagCompound tag)
    {
        size = tag.getByte("size");
        empty();
        NBTTagList nbttaglist = tag.getTagList("Items");
        for(int i = 0; i < nbttaglist.tagCount(); i++)
        {
            NBTTagCompound nbttagcompound1 = (NBTTagCompound) nbttaglist.tagAt(i);
            int slot = nbttagcompound1.getByte("Slot") & 0xff;
            if(slot >= 0 && slot < items.length)
            {
                items[slot] = ItemStack.loadItemStackFromNBT(nbttagcompound1);
            }
        }
        if(size != configSize)
            alignSize();
    }

    private void alignSize()
    {
        if(configSize > size)
        {
            ItemStack[] newItems = new ItemStack[sizes[configSize]];
            System.arraycopy(items, 0, newItems, 0, items.length);
            items = newItems;
            size = configSize;
            onInventoryChanged();
        }
        else
        {
            int numStacks = 0;
            for(ItemStack item : items)
                if(item != null)
                    numStacks++;

            if(numStacks <= sizes[configSize])
            {
                ItemStack[] newItems = new ItemStack[sizes[configSize]];
                int copyTo = 0;
                for(ItemStack item : items)
                {
                    if(item != null)
                    {
                        newItems[copyTo] = item;
                        copyTo++;
                    }
                }
                items = newItems;
                size = configSize;
                onInventoryChanged();
            }
        }
    }

    @Override
    public String type()
    {
        return "item";
    }

    public NBTTagCompound saveToTag()
    {
        if(size != configSize && open == 0)
            alignSize();

        NBTTagList nbttaglist = new NBTTagList();
        for(int slot = 0; slot < items.length; slot++)
        {
            if(items[slot] != null)
            {
                NBTTagCompound nbttagcompound1 = new NBTTagCompound();
                nbttagcompound1.setByte("Slot", (byte) slot);
                items[slot].writeToNBT(nbttagcompound1);
                nbttaglist.appendTag(nbttagcompound1);
            }
        }

        NBTTagCompound compound = new NBTTagCompound();
        compound.setTag("Items", nbttaglist);
        compound.setByte("size", (byte) size);

        return compound;
    }

    public ItemStack getStackInSlot(int slot)
    {
        synchronized(this)
        {
            return items[slot];
        }
    }

    public ItemStack getStackInSlotOnClosing(int slot)
    {
        synchronized(this)
        {
            return InventoryUtils.getStackInSlotOnClosing(this, slot);
        }
    }

    public void setInventorySlotContents(int slot, ItemStack stack)
    {
        synchronized(this)
        {
            items[slot] = stack;
            onInventoryChanged();
        }
    }

    public void openChest()
    {
        if(manager.client)
            return;

        synchronized(this)
        {
            open++;
            if(open == 1)
                EnderStorageSPH.sendOpenUpdateTo(null, owner, freq, true);
        }
    }

    public void closeChest()
    {
        if(manager.client)
            return;

        synchronized(this)
        {
            open--;
            if(open == 0)
                EnderStorageSPH.sendOpenUpdateTo(null, owner, freq, false);
        }
    }

    public int getNumOpen()
    {
        return open;
    }

    @Override
    public int getSizeInventory()
    {
        return sizes[size];
    }

    public ItemStack decrStackSize(int slot, int size)
    {
        synchronized(this)
        {
            return InventoryUtils.decrStackSize(this, slot, size);
        }
    }

    @Override
    public String getInvName()
    {
        return null;
    }

    @Override
    public int getInventoryStackLimit()
    {
        return 64;
    }

    @Override
    public void onInventoryChanged()
    {
        setDirty();
    }

    @Override
    public boolean isUseableByPlayer(EntityPlayer var1)
    {
        return true;
    }

    public void empty()
    {
        items = new ItemStack[getSizeInventory()];
    }
    
    public void openSMPGui(EntityPlayer player, final String name)
    {
        ServerUtils.openSMPContainer((EntityPlayerMP) player, new ContainerEnderItemStorage(player.inventory, this, false), new IGuiPacketSender()
        {
            @Override
            public void sendPacket(EntityPlayerMP player, int windowId)
            {
                PacketCustom packet = new PacketCustom(EnderStorageSPH.channel, 2);
                packet.writeByte(windowId);
                packet.writeString(owner);
                packet.writeShort(freq);
                packet.writeString(name);
                packet.writeByte(size);
                
                packet.sendToPlayer(player);
            }
        });
    }
    
    public int getSize()
    {
        return size;
    }

    public int openCount()
    {
        return open;
    }

    public void setClientOpen(int i)
    {
        if(manager.client)
            open = i;
    }
    
    @SideOnly(Side.CLIENT)
    public void openClientGui(int windowID, InventoryPlayer playerInv, String name, int size)
    {
        this.size = size;
        empty();
        ClientUtils.openSMPGui(windowID, new GuiEnderItemStorage(playerInv, this, name));
    }
    
    @Override
    public boolean isStackValidForSlot(int slot, ItemStack stack)
    {
        return true;
    }
    
    @Override
    public boolean isInvNameLocalized()
    {
        return true;
    }
}