package codechicken.enderstorage.storage.liquid;

import codechicken.core.liquid.ExtendedLiquidTank;
import codechicken.core.liquid.LiquidUtils;
import codechicken.enderstorage.api.AbstractEnderStorage;
import codechicken.enderstorage.api.EnderStorageManager;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraftforge.common.ForgeDirection;
import net.minecraftforge.liquids.ILiquidTank;
import net.minecraftforge.liquids.ITankContainer;
import net.minecraftforge.liquids.LiquidStack;

public class EnderLiquidStorage extends AbstractEnderStorage implements ITankContainer
{
    private class Tank extends ExtendedLiquidTank
    {
        public Tank(int capacity)
        {
            super(capacity);
        }

        @Override
        public void onLiquidChanged()
        {
            setDirty();
        }
    }
    
    private Tank tank;

    public EnderLiquidStorage(EnderStorageManager manager, String owner, int freq)
    {
        super(manager, owner, freq);
        tank = new Tank(16*LiquidUtils.B);
    }

    public void loadFromTag(NBTTagCompound tag)
    {
        tank.fromTag(tag.getCompoundTag("tank"));
    }

    @Override
    public String type()
    {
        return "liquid";
    }

    public NBTTagCompound saveToTag()
    {
        NBTTagCompound compound = new NBTTagCompound();
        compound.setTag("tank", tank.toTag());

        return compound;
    }

    @Override
    public int fill(ForgeDirection from, LiquidStack resource, boolean doFill)
    {
        return fill(0, resource, doFill);
    }

    @Override
    public int fill(int tankIndex, LiquidStack resource, boolean doFill)
    {
        return tank.fill(resource, doFill);
    }

    @Override
    public LiquidStack drain(ForgeDirection from, int maxDrain, boolean doDrain)
    {
        return drain(0, maxDrain, doDrain);
    }

    @Override
    public LiquidStack drain(int tankIndex, int maxDrain, boolean doDrain)
    {
        return tank.drain(maxDrain, doDrain);
    }

    @Override
    public ILiquidTank[] getTanks(ForgeDirection direction)
    {
        return new ILiquidTank[]{tank};
    }

    @Override
    public ILiquidTank getTank(ForgeDirection direction, LiquidStack type)
    {
        return tank;
    }

    public LiquidStack getLiquid()
    {
        return tank.getLiquid();
    }
}