package codechicken.enderstorage.storage.liquid;

import java.util.List;

import thermalexpansion.api.tileentity.IReconfigurableFacing;

import net.minecraft.block.Block;
import net.minecraft.entity.EntityLiving;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.tileentity.TileEntity;
import net.minecraftforge.common.ForgeDirection;
import net.minecraftforge.liquids.ILiquidTank;
import net.minecraftforge.liquids.ITankContainer;
import net.minecraftforge.liquids.LiquidStack;
import codechicken.core.alg.MathHelper;
import codechicken.core.asm.InterfaceDependancies;
import codechicken.core.liquid.LiquidUtils;
import codechicken.core.packet.PacketCustom;
import codechicken.core.raytracer.IndexedCuboid6;
import codechicken.core.vec.Cuboid6;
import codechicken.core.vec.Rotation;
import codechicken.core.vec.Scale;
import codechicken.core.vec.Transformation;
import codechicken.core.vec.Translation;
import codechicken.core.vec.Vector3;
import codechicken.enderstorage.api.EnderStorageManager;
import codechicken.enderstorage.common.TileFrequencyOwner;
import codechicken.enderstorage.internal.EnderStorageSPH;
import codechicken.enderstorage.storage.liquid.TankSynchroniser.TankState;

import static codechicken.core.vec.Vector3.*;

@InterfaceDependancies
public class TileEnderTank extends TileFrequencyOwner implements ITankContainer, IReconfigurableFacing
{
    public class EnderTankState extends TankState
    {
        @Override
        public void sendSyncPacket()
        {
            PacketCustom packet = new PacketCustom(EnderStorageSPH.channel, 5);
            packet.writeCoord(xCoord, yCoord, zCoord);
            packet.writeLiquidStack(s_liquid);
            packet.sendToChunk(worldObj, xCoord>>4, zCoord>>4);
        }
        
        @Override
        public void onLiquidChanged()
        {
            worldObj.updateAllLightTypes(xCoord, yCoord, zCoord);
        }
    }
    
    public class PressureState
    {        
        public boolean invert_redstone;
        public boolean a_pressure;
        public boolean b_pressure;
        
        public double a_rotate;
        public double b_rotate;
        
        public void update(boolean client)
        {
            if(client)
            {
                b_rotate = a_rotate;
                a_rotate = MathHelper.approachExp(a_rotate, approachRotate(), 0.5, 20);
            }
            else
            {
                b_pressure = a_pressure;
                a_pressure = worldObj.isBlockIndirectlyGettingPowered(xCoord, yCoord, zCoord) != invert_redstone;
                if(a_pressure != b_pressure)
                    sendSyncPacket();
            }
        }
        
        public double approachRotate()
        {
            return a_pressure ? -90 : 90;
        }

        private void sendSyncPacket()
        {
            PacketCustom packet = new PacketCustom(EnderStorageSPH.channel, 6);
            packet.writeCoord(xCoord, yCoord, zCoord);
            packet.writeBoolean(a_pressure);
            packet.sendToChunk(worldObj, xCoord>>4, zCoord>>4);
        }

        public void invert()
        {
            invert_redstone = !invert_redstone;
            worldObj.getChunkFromBlockCoords(xCoord, zCoord).setChunkModified();
        }
    }
    
    private static Cuboid6[] selectionBoxes = new Cuboid6[4];
    public static Transformation[] buttonT = new Transformation[3];
    static
    {
        for(int i = 0; i < 3; i++)
        {
            buttonT[i] = new Scale(0.6).with(new Translation(0.35+i*0.15, 0.91, 0.5));
            selectionBoxes[i] = selection_button.copy().transform(buttonT[i]);
        }
        selectionBoxes[3] = new Cuboid6(0.358, 0.268, 0.05, 0.662, 0.565, 0.15);
    }
    
    public int rotation;
    public EnderTankState liquid_state = new EnderTankState();
    public PressureState pressure_state = new PressureState();
    
    private EnderLiquidStorage storage;
    private boolean described;
    
    @Override
    public void updateEntity()
    {
        super.updateEntity();
        
        pressure_state.update(worldObj.isRemote);
        if(pressure_state.a_pressure)
            ejectLiquid();
        
        liquid_state.update(worldObj.isRemote);
    }
    
    private void ejectLiquid()
    {
        for(ForgeDirection side : ForgeDirection.VALID_DIRECTIONS)
        {
            TileEntity t = worldObj.getBlockTileEntity(xCoord+side.offsetX, yCoord+side.offsetY, zCoord+side.offsetZ);
            if (!(t instanceof ITankContainer))
                continue;
            
            ITankContainer c = (ITankContainer)t;
            LiquidStack liquid = drain(null, 100, false);
            if(liquid == null)
                continue;
            int qty = c.fill(side.getOpposite(), liquid, true);
            if(qty > 0)
                drain(null, qty, true);
        }
    }

    public void reloadStorage()
    {
        storage = (EnderLiquidStorage) EnderStorageManager.instance(worldObj.isRemote).getStorage(owner, freq, "liquid");
        if(!worldObj.isRemote)
            liquid_state.reloadStorage(storage);
    }
    
    @Override
    public EnderLiquidStorage getStorage()
    {
        return storage;
    }

    @Override
    public int fill(ForgeDirection from, LiquidStack resource, boolean doFill)
    {
        return storage.fill(from, resource, doFill);
    }

    @Override
    public int fill(int tankIndex, LiquidStack resource, boolean doFill)
    {
        return storage.fill(tankIndex, resource, doFill);
    }

    @Override
    public LiquidStack drain(ForgeDirection from, int maxDrain, boolean doDrain)
    {
        return storage.drain(from, maxDrain, doDrain);
    }

    @Override
    public LiquidStack drain(int tankIndex, int maxDrain, boolean doDrain)
    {
        return storage.drain(tankIndex, maxDrain, doDrain);
    }

    @Override
    public ILiquidTank[] getTanks(ForgeDirection direction)
    {
        return storage.getTanks(direction);
    }

    @Override
    public ILiquidTank getTank(ForgeDirection direction, LiquidStack type)
    {
        return storage.getTank(direction, type);
    }
    
    @Override
    public void onPlaced(EntityLiving entity)
    {
        rotation = (int)Math.floor(entity.rotationYaw * 4 / 360 + 2.5D) & 3;
        pressure_state.b_rotate = pressure_state.a_rotate = pressure_state.approachRotate();
    }
    
    @Override
    public void writeToNBT(NBTTagCompound tag)
    {
        super.writeToNBT(tag);
        tag.setByte("rot", (byte) rotation);
        tag.setBoolean("ir", pressure_state.invert_redstone);
    }
    
    @Override
    public void readFromNBT(NBTTagCompound tag)
    {
        super.readFromNBT(tag);
        rotation = tag.getByte("rot");
        pressure_state.invert_redstone = tag.getBoolean("ir");
    }
    
    @Override
    public void writeToPacket(PacketCustom packet)
    {
        packet.writeByte(rotation);
        packet.writeLiquidStack(liquid_state.s_liquid);
        packet.writeBoolean(pressure_state.a_pressure);
    }

    @Override
    public void handleDescriptionPacket(PacketCustom desc)
    {
        super.handleDescriptionPacket(desc);
        rotation = desc.readUnsignedByte();
        liquid_state.s_liquid = desc.readLiquidStack();
        pressure_state.a_pressure = desc.readBoolean();
        if(!described)
        {
            liquid_state.c_liquid = liquid_state.s_liquid;
            pressure_state.b_rotate = pressure_state.a_rotate = pressure_state.approachRotate();
        }
        described = true;
    }
    
    @Override
    public boolean activate(EntityPlayer player, int subHit)
    {
        if(subHit == 4)
        {
            pressure_state.invert();
            return true;
        }
        return LiquidUtils.fillTankWithContainer(this, player) || 
            LiquidUtils.emptyTankIntoContainer(this, player, storage.getLiquid());
    }
    
    @Override
    public void addTraceableCuboids(List<IndexedCuboid6> cuboids)
    {
        Vector3 pos = new Vector3(xCoord, yCoord, zCoord);
        cuboids.add(new IndexedCuboid6(0, new Cuboid6(0.15, 0, 0.15, 0.85, 0.916, 0.85).add(pos)));
        
        for(int i = 0; i < 4; i++)
            cuboids.add(new IndexedCuboid6(i+1, selectionBoxes[i].copy()
                    .transform(Rotation.quarterRotations[(rotation+2)%4].at(center)).add(pos)));
    }
    
    @Override
    public int getLightValue()
    {
        if(liquid_state.s_liquid.amount > 0 && liquid_state.c_liquid.itemID < Block.blocksList.length)
            return Block.lightValue[liquid_state.c_liquid.itemID];
        return 0;
    }
    
    @Override
    public boolean redstoneInteraction()
    {
        return true;
    }

    public void sync(PacketCustom packet)
    {
        if(packet.getType() == 5)
            liquid_state.sync(packet.readLiquidStack());
        else if(packet.getType() == 6)
            pressure_state.a_pressure = packet.readBoolean();
    }
    
    @Override
    public int getFacing()
    {
        return 0;
    }
    
    @Override
    public boolean setFacing(int side)
    {
        return false;
    }
    
    @Override
    public boolean rotateBlock()
    {
        rotation = (rotation+1)%4;
        PacketCustom.sendToChunk(getDescriptionPacket(), worldObj, xCoord>>4, zCoord>>4);
        return true;
    }
    
    @Override
    public boolean setFacing(ForgeDirection side)
    {
        return false;
    }
    
    @Override
    public int comparatorInput()
    {
        ILiquidTank tank = storage.getTank(null, null);
        return tank.getLiquid().amount*14/tank.getCapacity() + (tank.getLiquid().amount > 0 ? 1 : 0);
    }
}
